unit PerfDataComp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,
  PDH;


type
  TPerfCounter = class (TCollectionItem)
  public
    hCounter : Cardinal;
    Format : DWord;
    Name : String;
  end;

  TPerfQuery = class (TCollection)
  public
    Query : DWORD;
    UserData : DWORD;
    function AddCounter (AName: ansiString; AFormat : DWORD) : TPerfCounter;
    destructor Destroy; override;
  end;

  TPerfDataComp = class(TComponent)
  private
    FPerfQuery: TPerfQuery;
    FUserData : Cardinal;
    FBrowseDlgData: PDH_BROWSE_DLG_CONFIG_A;
    FActive : Boolean;
    FOutPut : TStrings;
    function GetActive: boolean;
    procedure SetActive(const Value: boolean);
  protected
    function NowStr : String;
    procedure RecordMsg (Msg  :string);
  public
    procedure Update;
    procedure BrowseCounters;
  published
    property OutPut : TStrings read FOutPut write FOutPut;
    property Active : boolean read GetActive write SetActive;
    property PerfQuery : TPerfQuery read FPerfQuery write FPerfQuery;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

procedure Register;

implementation

// '\\MARK\Process(Project1)\% Processor Time'

procedure Register;
begin
  RegisterComponents('VML', [TPerfDataComp]);
end;


{ TPerfData }


function PDHCallBack1 (Arg : DWORD) : PDH_STATUS;  stdcall;
var
  ThePerfDataComp : TPerfDataComp;
begin
  ThePerfDataComp := TPerfDataComp(Pointer(Arg));
  PDHCheck (ThePerfDataComp.FBrowseDlgData.CallBackStatus);
  ThePerfDataComp.PerfQuery.AddCounter (ThePerfDataComp.FBrowseDlgData.szReturnPathBuffer, PDH_FMT_DOUBLE);
  Result := ERROR_SUCCESS;
end;

constructor TPerfDataComp.Create(AOwner: TComponent);
begin
  inherited;
  FPerfQuery := TPerfQuery.Create (TPerfCounter);
  PDHCheck(PdhOpenQuery (nil, FUserData, FPerfQuery.Query));
end;

destructor TPerfDataComp.Destroy;
begin
  FPerfQuery.Free;
  PDHCheck(PdhCloseQuery(FPerfQuery.Query));
  inherited;
end;

function TPerfDataComp.GetActive: boolean;
begin
  Result := FActive;
end;

function TPerfDataComp.NowStr: String;
begin
  Result := FormatDateTime ('yyyy,mm,dd hh:mm:ss:ss', now);
end;

procedure TPerfDataComp.RecordMsg(Msg: string);
begin
  OutPut.Add (Self.Name + ': ' + NowStr +' ' + Msg);
end;

procedure TPerfDataComp.SetActive(const Value: boolean);
begin
  if (not(csLoading in componentState)) and (FOutPut = nil) then
    raise Exception.Create ('Need to set OutPut before activating');

  OutPut.Add (Self.Name + ': Activated at ' + NowStr);
  FActive := Value;
end;

procedure TPerfDataComp.Update;
var
  i : integer;
  RawValue : PDH_RAW_COUNTER;
  RawVal : PPDH_RAW_COUNTER;
  FormattedValue : PDH_FMT_COUNTERVALUE;
  PerfCounter : TPerfCounter;
begin

  PDHCheck(PdhCollectQueryData(FPerfQuery.Query));
  RawVal := @RawValue;

  for i := 0 to FPerfQuery.Count -1 do
  begin
    PerfCounter := FPerfQuery.Items[i] as TPerfCounter;
    PDHCheck(PdhGetRawCounterValue(PerfCounter.hCounter, @FUserData, RawVal));
    PDHCheck(PdhGetFormattedCounterValue(PerfCounter.hCounter, PerfCounter.Format, @FUserData, FormattedValue));
    PDHCheck(FormattedValue.CStatus);
    case PerfCounter.Format of
      PDH_FMT_LONG : RecordMsg (PerfCounter.Name + ': ' + IntToStr(FormattedValue.longValue));
      PDH_FMT_DOUBLE : RecordMsg (PerfCounter.Name + ': ' + floatToStrF(FormattedValue.doubleValue, ffFixed, 9, 2));
    end;
  end;
end;

procedure TPerfDataComp.BrowseCounters;
var
  ReturnPath: shortstring;
begin
  SetLength (ReturnPath, 255);
  FBrowseDlgData.szReturnPathBuffer := @ReturnPath;
  FBrowseDlgData.cchReturnPathLength := 255;
  FBrowseDlgData.dwCallBackArg := Integer(pointer(self));

  FBrowseDlgData.pCallBack := @PDHCallBack1;
  FBrowseDlgData.flags := [bHideDetailBox, bDisableMachineSelection];

  FBrowseDlgData.dwDefaultDetailLevel := 3;
  FBrowseDlgData.hWndOwner := Application.Handle;
  FBrowseDlgData.szDialogBoxCaption := 'Pick a counter';

  PDHCheck(PdhBrowseCountersA (@FBrowseDlgData));
end;

{ TCounters }

function TPerfQuery.AddCounter (AName: AnsiString; AFormat : DWORD): TPerfCounter;
var
  lCounter: HCOUNTER;
begin
  Result := Add as TPerfCounter;
  Result.Name := AName;
  PDHCheck(PdhAddCounterA (Query, Pchar(Aname), UserData, lCounter));
  Result.hCounter := lCounter;
  Result.Format := AFormat;
end;


destructor TPerfQuery.Destroy;
var
  i : integer;
  Counter : TPerfCounter;
begin
  for i := 0 to Count-1 do
  begin
    Counter := Items[i] as TPerfCounter;
    PDHCheck(PdhRemoveCounter (Counter.hCounter));
  end;
  inherited;
end;

end.
