unit PerfDataComp;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls,PDH;

type

  TPerfCounter = class (TCollectionItem)
  // Define um contador como item de uma collection
  // Possui uma propriedade: collection que aponta para a Collection
  // a qual pertence
    public
    hCounter : Cardinal; // Handle para contador
    Format : DWord;      // Formato do contador conforme instruo PdhGetFormattedCounterValue
    Name : String;       // Nome do contador:  retornado pelo object browser
  end;

  // A query  uma coleo de descrio de contadores
  // Add e Clear so usados para adicionar e retirar itens da coleo
  TPerfQuery = class (TCollection)
  public
    Query : DWORD;     // Handle para a query retornado por PdhOpenQuery
    UserData : DWORD;  //
    // Adiciona um contador  query. Tipos:
    // inteiro de 32-bits       (PDH_FMT_LONG)
    // inteiro de 64 bits       (PDH_FMT_LARGE)
    // double                   (PDH_FMT_DOUBLE).
    function AddCounter (AName: ansiString; AFormat : DWORD) : TPerfCounter;
    destructor Destroy; override;
  end;

  TPerfDataComp = class(TComponent)
  private
    FPerfQuery: TPerfQuery;     // Salva handle para a query
    FUserData : Cardinal;       // Dados do usurio a serem associados  query em PdhOpenQuery
    FBrowseDlgData: PDH_BROWSE_DLG_CONFIG_A;  // Dados para configurar o Browser de contadores
    FActive : Boolean;          // Estado: TRUE = ATIVO
    FOutPut : TStrings;         // Lista de strings de sada para exibio
    function GetActive: boolean;// Le estado do componente se ativo ou no
    procedure SetActive(const Value: boolean); // Seta estado do componente como ativo (TRUE) ou no
  protected
    function NowStr : String;   // Retorna data e hora correntes
    procedure RecordMsg (Msg  :string); // Salva mensagem na lista de sada
  public
    procedure Update;           // Atualiza a query
    procedure BrowseCounters;   // Exibe dilogo para escolha de contadores
  published
    // Lista de strings de sada
    property OutPut : TStrings read FOutPut write FOutPut;
    // Flag: componente ativo
    property Active : boolean read GetActive write SetActive;
    // Realiza uma query
    property PerfQuery : TPerfQuery read FPerfQuery write FPerfQuery;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

procedure Register;

implementation

// '\\MARK\Process(Project1)\% Processor Time'

procedure Register;   // Registra componente e o associa  aba VML
begin
  RegisterComponents('VML', [TPerfDataComp]);
end;


{ TPerfData ============================================================}

constructor TPerfDataComp.Create(AOwner: TComponent);
begin
  inherited;
  // Cria a query como uma coleo de counters (TPerfCounter)
  FPerfQuery := TPerfQuery.Create(TPerfCounter);
  // Abre a query e testa retorno, setando uma exceo se algo errado acontece
  // Retorna handle para a query no ltimo parmetro
  PDHCheck(PdhOpenQuery (nil, FUserData, FPerfQuery.Query));
end;

destructor TPerfDataComp.Destroy;
begin
  FPerfQuery.Free;  // Por que libera antes de fechar a query ???
  PDHCheck(PdhCloseQuery(FPerfQuery.Query)); // Encerra a query e testa retorno
  inherited;
end;

// Retorna data e hora correntes no formato desejado
function TPerfDataComp.NowStr: String;
begin
  Result := FormatDateTime ('yyyy,mm,dd hh:mm:ss:ss', now);
end;

// Adiciona um string ao final da lista de sada no formato:
// NomeDoComponente: Data:Hora Mensagem
// O Nome do componente ser definido pelo usurio
// PerfMon: 1999,9,13:12:20:20 Process[DemoPrj%ProcessorTime: 1.00
procedure TPerfDataComp.RecordMsg(Msg: string);
begin
  OutPut.Add(Self.Name + ': ' + NowStr +' ' + Msg);
end;

// Retorna estado do componente se ativo ou no
function TPerfDataComp.GetActive: boolean;
begin
  Result := FActive;
end;

// Define estado do componente conforme valor passado como parmetro
// O campo OutPut deve ser definido antes da funo ser chamada
procedure TPerfDataComp.SetActive(const Value: boolean);
begin
  // Se componente j estiver carregado e Output indefinido ...
  if (not(csLoading in componentState)) and (FOutPut = nil) then
    raise Exception.Create ('Need to set OutPut before activating');

  OutPut.Add (Self.Name + ': Activated at ' + NowStr);
  FActive := Value;
end;

procedure TPerfDataComp.Update;
var
  i : integer;
  RawValue : PDH_RAW_COUNTER;   // Estrutura para receber valor do contador
  RawVal : PPDH_RAW_COUNTER;    // Apontador para PDH_RAW_COUNTER
  FormattedValue : PDH_FMT_COUNTERVALUE;   // Estrutura para receber valor formatado do contador
  PerfCounter : TPerfCounter;
begin
  // Realiza leitura dos contadores
  PDHCheck(PdhCollectQueryData(FPerfQuery.Query));
  RawVal := @RawValue;          // RawValue contm o endereo de RawValue

  for i := 0 to FPerfQuery.Count -1 do  // Para todos os contadores da query faa
  begin
    PerfCounter := FPerfQuery.Items[i] as TPerfCounter;
    // Le valor bruto do contador para RawValue: handle, @buffer_tipo_de_dado, @buffer_valor
    PDHCheck(PdhGetRawCounterValue(PerfCounter.hCounter, @FUserData, RawVal));
    // Le valor formatado da varivel para FormattedValue
    PDHCheck(PdhGetFormattedCounterValue(PerfCounter.hCounter, PerfCounter.Format, @FUserData, FormattedValue));
    PDHCheck(FormattedValue.CStatus);
    // Imprime valor formatado
    // Valor bruto foi buscado, mas no foi impresso
    case PerfCounter.Format of
      PDH_FMT_LONG : RecordMsg (PerfCounter.Name + ': ' + IntToStr(FormattedValue.longValue));
      PDH_FMT_DOUBLE : RecordMsg (PerfCounter.Name + ': ' + floatToStrF(FormattedValue.doubleValue, ffFixed, 9, 2));
    end;
  end;
end;


// Funo callback associada ao browser de contadores
// Para cada contador definido, esta funo ser chamada
function PDHCallBack1 (Arg : DWORD) : PDH_STATUS;  stdcall;
var
  ThePerfDataComp : TPerfDataComp;
begin
  ThePerfDataComp := TPerfDataComp(Pointer(Arg));
  PDHCheck (ThePerfDataComp.FBrowseDlgData.CallBackStatus); // testa status de retorno do browser
  // Adiciona um novo contador. Tipo = DOUBLE (funciona com todos os counters)
  // O string retornado pelo browser corresponde ao nome do contador
  ThePerfDataComp.PerfQuery.AddCounter(ThePerfDataComp.FBrowseDlgData.szReturnPathBuffer, PDH_FMT_DOUBLE);
  Result := ERROR_SUCCESS;
end;

procedure TPerfDataComp.BrowseCounters;
var
  ReturnPath: shortstring;
begin
  SetLength (ReturnPath, 255);
  // Prepara estrutura PDH_BROWSE_DLG_CONFIG
  FBrowseDlgData.szReturnPathBuffer := @ReturnPath;
  FBrowseDlgData.cchReturnPathLength := 255;
  // Define argumento para funo callback: ponteiro para objeto TPerfData
  FBrowseDlgData.dwCallBackArg := Integer(pointer(self));
  // Define endereo da funo callback
  FBrowseDlgData.pCallBack := @PDHCallBack1;
  // Desabilita seleo de outra mquina da rede
  FBrowseDlgData.flags := [bHideDetailBox, bDisableMachineSelection];
  // Define nvel de detalhe do dilogo:
  FBrowseDlgData.dwDefaultDetailLevel := 3;  // PERF_DETAIL_WIZARD
  FBrowseDlgData.hWndOwner := Application.Handle;
  FBrowseDlgData.szDialogBoxCaption := 'Escolha um contador';  // Ttulo da janela
  // Chama janela de dilogo
  PDHCheck(PdhBrowseCountersA (@FBrowseDlgData));
end;

{ TCounters ==============================================================}

function TPerfQuery.AddCounter (AName: AnsiString; AFormat : DWORD): TPerfCounter;
var
  lCounter: HCOUNTER;  // Recebe handle para o contador
begin
  Result := Add as TPerfCounter;
  Result.Name := AName;   // Nome do contador definido no browser
  PDHCheck(PdhAddCounterA (Query, Pchar(Aname), UserData, lCounter));
  Result.hCounter := lCounter;  // Salva handle retornado
  Result.Format := AFormat;
end;

 // Remove todos os contadores associados  query e termina
destructor TPerfQuery.Destroy;
var
  i : integer;
  Counter : TPerfCounter;
begin
  for i := 0 to Count-1 do
  // Para todos os contadores associados  query faa
  begin
    Counter := Items[i] as TPerfCounter;
    PDHCheck(PdhRemoveCounter (Counter.hCounter));
  end;
  inherited;
end;

end.
